'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var useLocalStorage = _interopDefault(require('react-use/lib/useLocalStorage'));
var PropTypes = _interopDefault(require('prop-types'));

/* eslint-env browser */

const getFromPath = (data, path) => path.split('.').reduce((curr, sub) => curr && curr[sub], data);

const Context = React.createContext();
const messageProxy = {
  set() {
    throw new Error('[use-locales] messages is unvariant.');
  },

  get(obj, prop) {
    return obj[prop] || prop;
  }

};

const LocalesProvider = ({
  children,
  loadingElm,
  hashKey
}) => {
  const [lang, setLang] = useLocalStorage('lang', undefined);
  const [locales, setLocales] = useLocalStorage('locales', undefined);
  React.useEffect(() => {
    // load lang
    let currentLang = lang;

    if (!lang) {
      // retrieve lang from navigator
      const {
        navigator
      } = window;

      if (navigator) {
        const {
          language,
          userLanguage,
          languages
        } = navigator;
        currentLang = language || userLanguage || languages && languages.length && languages[0] || 'en';
        if (currentLang.length > 2) currentLang = currentLang.substring(0, 2);
        if (currentLang.length < 2) currentLang = 'en';
      }

      setLang(currentLang);
    } // load locales


    if (!locales || !locales.date || !locales.lang || locales.lang !== currentLang || hashKey && hashKey !== locales.hashKey || Date.now() - locales.date > 259200000
    /* 3 days */
    ) {
        const loadLocales = fetchLang => fetch(`${process.env.PUBLIC_URL}/locales/${fetchLang}.json${hashKey ? `?hashKey=${hashKey}` : ''}`).then(raw => raw.json()).then(newLocales => setLocales({
          data: newLocales,
          lang: fetchLang,
          date: Date.now(),
          hashKey
        }));

        loadLocales(currentLang).catch(() => {
          setLang('en');
          loadLocales('en');
        });
      }
  }, [hashKey]);
  const getMessages = React.useCallback((path = '') => {
    let {
      data: messages
    } = locales;
    if (path !== '') messages = getFromPath(locales.data, path);
    if (typeof Proxy !== 'undefined') return new Proxy(messages, messageProxy);
    return messages;
  }, [locales]);
  if (!locales) return loadingElm;
  return React__default.createElement(Context.Provider, {
    value: {
      lang,
      locales,
      getMessages
    }
  }, children);
};

LocalesProvider.propTypes = {
  children: PropTypes.node.isRequired,
  loadingElm: PropTypes.node,
  hashKey: PropTypes.oneOfType(PropTypes.string, PropTypes.number)
};
LocalesProvider.defaultProps = {
  loadingElm: null,
  hashKey: null
};

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

// specs: http://userguide.icu-project.org/formatparse/messages
// inspiration: https://github.com/formatjs/formatjs/tree/master/packages/intl-messageformat
// why redoing it? => bundle size & curiosity!
// this regexp is used to get all pieces {}
const REGEXP_MESSAGE = /{(.+?)}+/g; // this regexp is used to remove brackets from a piece

const REGEXP_PIECE = /[{}]/g;

function cleanPiece(piece) {
  return piece && piece.replace(REGEXP_PIECE, '').trim();
}

function format(message, data, lang = 'en') {
  if (!message || message.trim().length === 0) return message;
  const pieces = message.match(REGEXP_MESSAGE);
  if (!pieces) return message;
  return pieces.reduce((newMessage, piece) => {
    // , means we have a key with options (eg: {birthday, date})
    // in this case we don't have options, so this is simple, we replace key by the value
    if (!piece.includes(',')) return newMessage.replace(piece, data[cleanPiece(piece)] || ''); // here we have options, we parse the type of options
    // knowing they key is always the first argument, eg in "{birthday, date}" the value key is 'birthday'
    // the second argument is type of options, in previous example this is 'date'

    const pieceOptions = piece.split(',');
    const key = cleanPiece(pieceOptions[0]);
    if (!data[key]) return newMessage.replace(piece, '');
    const type = cleanPiece(pieceOptions[1]);

    switch (type) {
      case 'date':
        {
          // TODO: cache formater
          const value = new Intl.DateTimeFormat(lang).format(new Date(data[key]));
          return newMessage.replace(piece, value);
        }

      case 'currency':
        {
          const currency = cleanPiece(pieceOptions[2]); // TODO: cache formater

          const value = new Intl.NumberFormat(lang, {
            style: 'currency',
            currency
          }).format(data[key]);
          return newMessage.replace(piece, value);
        }

      case 'number':
        {
          const value = new Intl.NumberFormat(lang).format(data[key]);
          return newMessage.replace(piece, value);
        }

      default:
        {
          const error = new Error('messageFormat - Unknown type');
          error.type = type;
          throw error;
        }
    }
  }, message);
}

const useLocales = (path, object) => {
  const firstDraw = React.useRef(true);
  const {
    getMessages,
    lang
  } = React.useContext(Context);
  const [messages, setMessages] = React.useState(getMessages(path));
  React.useLayoutEffect(() => {
    const rawMessage = getMessages(path);
    if (firstDraw.current && !object) return;
    firstDraw.current = false;

    if (!object) {
      setMessages(rawMessage);
      return;
    }

    setMessages(Object.entries(rawMessage).reduce((acc, [key, value]) => _objectSpread({}, acc, {
      [key]: format(value, object, lang)
    }), {}));
  }, [object]);
  return messages;
};

const useLang = () => {
  const {
    lang
  } = React.useContext(Context);
  return lang;
};

exports.LocalesProvider = LocalesProvider;
exports.useLocales = useLocales;
exports.useLang = useLang;
//# sourceMappingURL=index.js.map
